/*****************************************************************************
*
* Atmel Corporation
*
* File              : main.c
* Compiler          : IAR EWAAVR 2.28a/3.10c
* Revision          : $Revision: 1.13 $
* Date              : $Date: 24. mai 2004 11:31:20 $
* Updated by        : $Author: ltwa $
*
* Support mail      : avr@atmel.com
*
* Supported devices : All devices with a TWI module can be used.
*                     The example is written for the ATmega16
*
* AppNote           : AVR315 - TWI Master Implementation
*
* Description       : Example of how to use the driver for TWI master 
*                     communication.
*                     This code reads PORTD and sends the status to the TWI slave.
*                     Then it reads data from the slave and puts the data on PORTB.
*                     To run the example use STK500 and connect PORTB to the LEDS,
*                     and PORTD to the switches. .
*
****************************************************************************/
#include <avr/io.h>
#ifndef F_CPU
#define F_CPU 8000000UL
#endif
#include <avr/delay.h>
#include <avr/interrupt.h>
#include <util/twi.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "df_all.h"
#include "TWI_Master.h"
#include "TWI_Function.h"

#define TWI_GEN_CALL         0x00  // The General Call address is 0

// Sample TWI transmission commands
#define TWI_CMD_MASTER_WRITE 0x10
#define TWI_CMD_MASTER_READ  0x20

// Sample TWI transmission states, used in the main application.
#define SEND_DATA             0x01
#define REQUEST_DATA          0x02
#define READ_DATA_FROM_BUFFER 0x03

unsigned char TWI_Act_On_Failure_In_Last_Transmission ( unsigned char TWIerrorMsg )
{                   // A failure has occurred, use TWIerrorMsg to determine the nature of the failure
                    // and take appropriate actions.
                    // Se header file for a list of possible failures messages.
                    // Here is a simple sample, where if received a NACK on the slave address,
                    // then a retransmission will be initiated.
 
	if ( (TWIerrorMsg == TWI_MTX_ADR_NACK) | (TWIerrorMsg == TWI_MRX_ADR_NACK) )
	TWI_Start_Transceiver();
    
	return TWIerrorMsg; 
}
void _enable_interrupt(void)
{	//TWCR|=(1<<TWIE);	// enable Interupt.
	sei();
}

void main(void)
{	unsigned char TWI_Puffer[6];
	unsigned char Daten[4];
	unsigned char TWI_targetSlaveAddress, TWI_operation=0;

	//LED feedback port - connect port B to the STK500 LEDS
	DDRC = 0xFF;
	PORTC = 0x00;

	DDRE = 0xC0;
	PORTE = 0xC0;
  
	//Switch port - connect portD to the STK500 switches
	DDRD  = 0x00;
	TWI_Master_Initialise();
	_enable_interrupt();
  
	TWI_targetSlaveAddress   = 0x50;

	// This example code runs forever; sends a byte to the slave, then requests a byte
	// from the slave and stores it on PORTB, and starts over again. Since it is interupt
	// driven one can do other operations while waiting for the transceiver to complete.
  
	// Send initial data to slave
	Daten[0] = 0x10;
	Daten[1] = 0x00;
	TWI_Puffer[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
	TWI_Puffer[1] = 0x50;
	TWI_Puffer[2] = 0xD0;
	TWI_Puffer[3] = 0x10;
	TWI_Puffer[4] = 0x55;
	TWI_Puffer[5] = 0xCC;
	TWI_Start_Transceiver_With_Data( TWI_Puffer, 6 );

	TWI_operation = REQUEST_DATA; // Set the next operation
	PORTC = 0x10;
	while(1)
	//{	for (i=0;i<5;i++)
		{	if( ! TWI_Transceiver_Busy())			// Check if the TWI Transceiver has completed an operation.
	    	{  if(TWI_statusReg.lastTransOK)    	// Check if the last operation was successful

	      		{	if	(TWI_operation == SEND_DATA) 	// Determine what action to take now, Send data to slave
	        		{ 	TWI_Puffer[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS)|(FALSE<<TWI_READ_BIT);
	          			TWI_Start_Transceiver_With_Data( TWI_Puffer, 6 );
	          			TWI_operation = REQUEST_DATA; // Set next operation
						PORTE = 0x80;
	        		}
	        		else if(TWI_operation == REQUEST_DATA)// Request data from slave
	        		{	TWI_Puffer[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS)|(TRUE<<TWI_READ_BIT);
						TWI_Start_Transceiver_With_Data( TWI_Puffer, 6 );
	          			TWI_operation = READ_DATA_FROM_BUFFER; // Set next operation        
						PORTE = 0x40;
	        		}
	        		else if (TWI_operation == READ_DATA_FROM_BUFFER)// Get the received data from the transceiver buffer
	        		{	TWI_Get_Data_From_Transceiver( TWI_Puffer, 6 );
	          			PORTC = 0x00;								//TWI_Puffer[6]; Store data on PORTB.
	          			TWI_operation = SEND_DATA;    				// Set next operation        
	        		}
	      		}
	      		else // Got an error during the last transmission
	      		{	// Use TWI status information to detemine cause of failure and take appropriate actions. 
	        		TWI_Act_On_Failure_In_Last_Transmission( TWI_Get_State_Info( ) );
	      		}
			}	// Do something else while waiting for the TWI Transceiver to complete the current operation
			TWI_ISR();
		}
		
//	}
}

